import json
from pywsgiapp.support.parseMultipartForm import ParseMultipartFormdata as pmf


class ParseEnviron:
    headers = {}

    def __init__(self, environ: dict):
        self.environ = environ
        self.headers = {}

    def getRequestHeaders(self):
        for key, value in self.environ.items():
            if key.startswith("HTTP_"):
                header_name = key[5:].replace("_", "-").title()
                self.headers[header_name] = value
            elif key in (
                "CONTENT_TYPE",
                "CONTENT_LENGTH",
                "REQUEST_METHOD",
                "QUERY_STRING",
                "RAW_URI",
                "PATH_INFO",
                "REMOTE_ADDR",
                "Authorization",
            ):
                self.headers[key] = value
        return self.headers

    def getUrl(self):
        return self.environ["PATH_INFO"][
            1:
        ]

    def getPostData(self):
        if "application/json" in self.getContentType():
            return self.extractJsonFromWsgiInput()

        elif "multipart/form-data" in self.getContentType():
            return self.extractFormFromWsgiInput()
        return None

    def getContentLength(self):
        return int(self.headers.get("CONTENT_LENGTH", 0))

    def getContentType(self):
        return self.environ["CONTENT_TYPE"]

    def extractJsonFromWsgiInput(self) -> dict:
        try:
            return json.loads(self.environ["wsgi.input"].read().decode("UTF-8"))
        except:
            return {}

    def extractFormFromWsgiInput(self) -> dict:
        try:
            unparsedData = self.environ["wsgi.input"].read()
            data = pmf(unparsedData).parse()
            return data
        except:
            return {}



import os
import re


class ParseMultipartFormdata:
    def __init__(self, raw_data):
        self.raw_data = raw_data

    def parse(self):
        return self.parseMultipartFormdata(self.raw_data)

    def getBoundary(self):
        first_crlf = self.raw_data.find(b'\r\n')
        if first_crlf == -1:
            return None
        boundary_line = self.raw_data[:first_crlf]
        boundary = boundary_line.lstrip(b'-')
        return b'--' + boundary

    def getParts(self):
        full_boundary = self.getBoundary()
        if not full_boundary:
            raise ValueError("Boundary not found in the raw data.")

        parts = self.raw_data.split(full_boundary)
        return parts

    def parseMultipartFormdata(self, raw_data):
        parts = self.getParts()
        result = {}

        for part in parts:
            part = part.strip(b'\r\n')
            if not part or part in (b'--', b'--\r\n'):
                continue

            headersEnd = part.find(b'\r\n\r\n')
            if headersEnd == -1:
                continue
            headersBlock = part[:headersEnd]
            contentBlock = part[headersEnd+4:]

            cdMatch = re.search(
                b'Content-Disposition:\\s*form-data;\\s*(.*)', headersBlock, re.IGNORECASE)
            if not cdMatch:
                continue
            cdData = cdMatch.group(1)

            nameMatch = re.search(b'name="([^"]+)"', cdData)
            filenameMatch = re.search(b'filename="([^"]+)"', cdData)
            if not nameMatch:
                continue

            field_name = nameMatch.group(1).decode('utf-8', 'replace')

            if filenameMatch:
                filename = filenameMatch.group(1).decode('utf-8', 'replace')
                result[field_name] = {
                    'filename': filename,
                    'content': contentBlock
                }
            else:
                result[field_name] = contentBlock.decode('utf-8', 'replace')

        return result



from pywsgiapp.WSGIApp import createWSGIApp  

def requestHandler(url:str, requestHeaders:dict, postData:dict) -> dict:
    
    response_body = f"Received URL: {url}, Headers: {requestHeaders}, Post Data: {postData}"
    return {
        "responseCode": 200,
        "responseHeaders": {"Content-Type": "text/plain"},
        "responseBody": response_body
    }

app = createWSGIApp(requestHandler)




import logging

from pywsgiapp.config.config import Config
from pywsgiapp.support.parseEnviron import ParseEnviron as Pe


class WsgiApp:

    def __init__(self):
        self.dConfig = Config()
        if self.dConfig.appLoggingStatus():
            logging.basicConfig(
                filename="logs/wsgiLog.log", level=logging.INFO)

    def processRequestData(self, environ):
        pe = Pe(environ)
        url = pe.getUrl()
        requestHeaders = pe.getRequestHeaders()
        if "CONTENT_TYPE" in requestHeaders:
            postData = pe.getPostData()
        else:
            postData = {}
        return url, requestHeaders, postData

    def processResponse(self, resp):
        statusCode = str(resp["responseCode"])
        responseHeader = resp["responseHeaders"]
        encodeResponse = str(resp["responseBody"]).encode("UTF-8")
        responseHeaderList = [(key, value)
                              for key, value in responseHeader.items()]
        return statusCode, responseHeaderList, encodeResponse

    def logInfo(self, data):
        logging.info(data)


def createWSGIApp(reqestHandler):
    """
    Create a WSGI application.

    Args:
        request_handler (function): A function that processes the request and returns a dictionary:
            Parameters:
            - url (str): The URL of the request.
            - requestHeaders (dict): A dictionary of request headers.
            - postData (dict): The parsed data from the request body
            Returns:
            - responseCode (int): The HTTP status code of the response.
            - responseHeaders (dict): A dictionary of headers to include in the response.
            - responseBody (str): The body of the response.

    Returns:
        function: A WSGI application callable.
    """
    def app(environ, start_response):
        wsgiapp = WsgiApp()
        url, requestHeaders, postData = wsgiapp.processRequestData(environ)
        resp = reqestHandler(url, requestHeaders, postData)
        statusCode, responseHeaderList, encodeResponse = wsgiapp.processResponse(
            resp)
        start_response(statusCode, responseHeaderList)
        return [encodeResponse]
    return app



from pywsgiapp.WSGIApp import createWSGIApp
import json


def requestHandler(url: str, requestHeaders: dict, postData: dict) -> dict:
    data = {
        "name": "John Doe",
        "age": 30,
        "phone": None,
        "is_student": False,
        "courses": ["Math", "Science", "History"],
        "address": {
            "street": "123 Main St",
            "city": "New York",
            "state": "NY"
        },
    }
    response_body = json.dumps(data) # This will be a JSON string 
    
    return {
        "responseCode": 200,
        "responseHeaders": {"Content-Type": "text/plain"},
        "responseBody": response_body
    }

app = createWSGIApp(requestHandler)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="pywsgiapp",
    version="0.0.4",
    description="Bring Your Code to the Web with Ease using a Lightweight WSGI Application.",
    long_description=long_description, 
    long_description_content_type="text/markdown",  
    author="Jay Thorat",
    author_email="dev.jaythorat@gmail.com",
    url="https://github.com/jaythorat/pywsgiapp",
    packages=find_packages(include=["pywsgiapp", "pywsgiapp.*"]),
    install_requires=["gunicorn>=23.0.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.12",
)



from pywsgiapp.WSGIApp import createWSGIApp  

def requestHandler(url:str, requestHeaders:dict, postData:dict) -> dict:
    """
        If a file is present in the 'postData', it can be accessed using the provided key.
        The filename is available under the key 'filename', and the file content can be accessed 
        using the key 'content'.
        Example:
        postData = {
            'file': {
            'filename': 'example.txt',
            'content': b'Hello, World!'
            }
        }
        """
    if postData and 'file' in postData:
        fileName = postData['file']['filename']
        fileContent = postData['file']['content']
        with open(fileName, 'wb') as f:
            f.write(fileContent)
        response_body = f"File '{fileName}' saved successfully."
    else:
        response_body = "No file data received."            
    
    return {
        "responseCode": 200,
        "responseHeaders": {"Content-Type": "text/plain"},
        "responseBody": response_body
    }

app = createWSGIApp(requestHandler)

